@@ -2,7 +2,7 @@
 /*
   Copyright (C) 2010  Tetsuro IKEDA
   Copyright (C) 2010-2013  Kentoku SHIBA
-  Copyright (C) 2011-2024  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2011-2025  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -35,10 +35,12 @@ extern "C" {
 #include <mrn_database.hpp>
 #include <mrn_buffers.hpp>
 
-#if __cplusplus >= 201402
-#  define mrn_override override
-#else
-#  define mrn_override
+#if !(defined(MRN_MARIADB_P) && MYSQL_VERSION_ID < 100600)
+#  define MRN_HANDLER_HAVE_TABLE_TYPE
+#endif
+
+#ifdef MRN_MARIADB_P
+#  define MRN_HANDLER_HAVE_INDEX_TYPE
 #endif
 
 #ifndef MRN_MARIADB_P
@@ -49,7 +51,7 @@ extern "C" {
 #  define MRN_HANDLER_HAVE_MULTI_RANGE_READ_INFO_KEY_PARTS
 #endif
 
-#if (MYSQL_VERSION_ID >= 100505) && defined(MRN_MARIADB_P)
+#ifdef MRN_MARIADB_P
 #  define MRN_HANDLER_DELETE_TABLE_HAVE_HTON_DROP_TABLE
 #endif
 
@@ -71,7 +73,7 @@ extern "C" {
 #  define MRN_NEED_FREE_STRING_MEMALLOC_PLUGIN_VAR
 #endif
 
-#if defined(MRN_MARIADB_P) || MYSQL_VERSION_ID < 80012
+#ifdef MRN_MARIADB_P
 #  define MRN_HAVE_HA_EXTRA_CACHE
 #  define MRN_HAVE_HA_EXTRA_NO_CACHE
 #  define MRN_HAVE_HA_EXTRA_KEY_CACHE
@@ -87,12 +89,7 @@ extern "C" {
 #  define MRN_HAVE_HA_EXTRA_PREPARE_FOR_FORCED_CLOSE
 #endif
 
-#if MYSQL_VERSION_ID >= 80002
-#  define MRN_HAVE_HA_EXTRA_BEGIN_ALTER_COPY
-#  define MRN_HAVE_HA_EXTRA_END_ALTER_COPY
-#endif
-
-#if (!defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 80002)
+#ifndef MRN_MARIADB_P
 #  define MRN_HAVE_HA_EXTRA_NO_AUTOINC_LOCKING
 #endif
 
@@ -100,6 +97,13 @@ extern "C" {
 #  define MRN_HAVE_HA_EXTRA_NO_READ_LOCKING
 #endif
 
+#if (defined(MRN_MARIADB_P) &&                                                 \
+     ((MYSQL_VERSION_ID >= 101112 && MYSQL_VERSION_ID < 101200) ||             \
+      (MYSQL_VERSION_ID >= 110406 && MYSQL_VERSION_ID < 110500) ||             \
+      (MYSQL_VERSION_ID >= 110802)))
+#  define MRN_HAVE_HA_EXTRA_ABORT_ALTER_COPY
+#endif
+
 #if (defined(MRN_MARIADB_P) &&                                                 \
      ((MYSQL_VERSION_ID >= 100600 && MYSQL_VERSION_ID < 100616) ||             \
       (MYSQL_VERSION_ID >= 101100 && MYSQL_VERSION_ID < 101106)))
@@ -150,6 +154,11 @@ extern "C" {
 
 #ifdef MRN_MARIADB_P
 #  define MRN_HANDLER_AUTO_REPAIR_HAVE_ERROR
+#  define MRN_HANDLER_AUTO_REPAIR_ERROR_OVERRIDE override
+#  define MRN_HANDLER_AUTO_REPAIR_VOID_OVERRIDE
+#else
+#  define MRN_HANDLER_AUTO_REPAIR_ERROR_OVERRIDE
+#  define MRN_HANDLER_AUTO_REPAIR_VOID_OVERRIDE override
 #endif
 
 #ifndef MRN_MARIADB_P
@@ -233,7 +242,7 @@ typedef uint mrn_alter_table_flags;
 #  define MRN_HANDLER_RECORDS_RETURN_ERROR
 #endif
 
-#if MYSQL_VERSION_ID < 80002 || defined(MRN_MARIADB_P)
+#ifdef MRN_MARIADB_P
 #  define MRN_HANDLER_HAVE_KEYS_TO_USE_FOR_SCANNING
 #endif
 
@@ -241,11 +250,11 @@ typedef uint mrn_alter_table_flags;
 #  define MRN_HANDLER_HAVE_TABLE_CACHE_TYPE
 #endif
 
-#if (!defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 80002)
+#ifndef MRN_MARIADB_P
 #  define MRN_ST_MYSQL_PLUGIN_HAVE_CHECK_UNINSTALL
 #endif
 
-#if (!defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 80002)
+#ifndef MRN_MARIADB_P
 #  define MRN_HANDLER_OPEN_HAVE_TABLE_DEFINITION
 #  define MRN_HANDLER_CREATE_HAVE_TABLE_DEFINITION
 #endif
@@ -256,7 +265,7 @@ typedef uint mrn_alter_table_flags;
 #  define MRN_HANDLER_TRUNCATE_HAVE_TABLE_DEFINITION
 #endif
 
-#if (!defined(MRN_MARIADB_P) && MYSQL_VERSION_ID >= 80002)
+#ifndef MRN_MARIADB_P
 #  define MRN_HANDLERTON_CREATE_HAVE_PARTITIONED
 #endif
 
@@ -300,27 +309,15 @@ typedef uint mrn_alter_table_flags;
 #  define MRN_HANDLER_HAVE_GET_FOREIGN_DUP_KEY
 #endif
 
-#if !defined(MRN_MARIADB_P) ||                                                 \
-  (defined(MRN_MARIADB_P) && MYSQL_VERSION_ID < 100500)
+#ifndef MRN_MARIADB_P
 #  define MRN_HANDLER_HAVE_NOTIFY_TABLE_CHANGED
 #  define MRN_HANDLER_HAVE_PRIMARY_KEY_IS_CLUSTERED
 #endif
 
-#if defined(HAVE_PSI_INTERFACE) &&                                             \
-  (MYSQL_VERSION_ID < 80002 || defined(MRN_MARIADB_P))
+#if defined(HAVE_PSI_INTERFACE) && defined(MRN_MARIADB_P)
 #  define MRN_HAVE_PSI_SERVER
 #endif
 
-#ifdef MRN_MARIADB_P
-typedef const uchar* mrn_key_copy_from_record_t;
-#else
-#  if MYSQL_VERSION_ID >= 80000
-using mrn_key_copy_from_record_t = const uchar*;
-#  else
-typedef uchar* mrn_key_copy_from_record_t;
-#  endif
-#endif
-
 #ifdef MRN_MARIADB_P
 typedef const uchar* mrn_update_row_new_data_t;
 #else
@@ -363,8 +360,8 @@ typedef uchar* mrn_write_row_buf_t;
 #endif
 
 #ifdef MRN_MARIADB_P
-#  define MRN_HANDLER_RESTORE_AUTO_INCREMENT_OVERRIDE mrn_override
-#  define MRN_HANDLER_RELEASE_AUTO_INCREMENT_OVERRIDE mrn_override
+#  define MRN_HANDLER_RESTORE_AUTO_INCREMENT_OVERRIDE override
+#  define MRN_HANDLER_RELEASE_AUTO_INCREMENT_OVERRIDE override
 #else
 #  define MRN_HANDLER_RESTORE_AUTO_INCREMENT_OVERRIDE
 #  define MRN_HANDLER_RELEASE_AUTO_INCREMENT_OVERRIDE
@@ -380,18 +377,17 @@ typedef uchar* mrn_write_row_buf_t;
 #  define MRN_HANDLER_HAVE_FOREIGN_KEY_INFO
 #endif
 
-#if MYSQL_VERSION_ID >= 100504 && defined(MRN_MARIADB_P)
+#ifdef MRN_MARIADB_P
 #  define MRN_HANDLER_RECORDS_IN_RANGE_HAVE_PAGE_RANGE
 #endif
 
-#if MYSQL_VERSION_ID < 80000 || defined(MRN_MARIADB_P)
+#ifdef MRN_MARIADB_P
 #  define MRN_HANDLER_NEED_OVERRIDE_UNBIND_PSI
 #  define MRN_HANDLER_NEED_OVERRIDE_REBIND_PSI
 #endif
 
 #if defined(MRN_MARIADB_P) &&                                                  \
-  ((MYSQL_VERSION_ID >= 100525 && MYSQL_VERSION_ID < 100600) ||                \
-   (MYSQL_VERSION_ID >= 100618 && MYSQL_VERSION_ID < 100700) ||                \
+  ((MYSQL_VERSION_ID >= 100618 && MYSQL_VERSION_ID < 100700) ||                \
    (MYSQL_VERSION_ID >= 101108 && MYSQL_VERSION_ID < 101200) ||                \
    (MYSQL_VERSION_ID >= 110402))
 #  define MRN_HANDLER_ENABLE_INDEXES_PARAMETERS       key_map map, bool persist
@@ -438,6 +434,26 @@ typedef uchar* mrn_write_row_buf_t;
       (key_info[i].flags & HA_NOSAME))
 #endif
 
+#if defined(MRN_MARIADB_P) && (MYSQL_VERSION_ID >= 110400)
+using mrn_io_and_cpu_cost = IO_AND_CPU_COST;
+#  define MRN_HANDLER_HAVE_MULTI_RANGE_READ_INFO_CONST_LIMIT
+#  define MRN_HANDLER_HAVE_KEYREAD_TIME
+#else
+using mrn_io_and_cpu_cost = double;
+#  define MRN_HANDLER_HAVE_READ_TIME
+#endif
+
+#if defined(MRN_MARIADB_P) &&                                                  \
+  ((MYSQL_VERSION_ID >= 100620 && MYSQL_VERSION_ID < 100700) ||                \
+   (MYSQL_VERSION_ID >= 101110 && MYSQL_VERSION_ID < 101200) ||                \
+   (MYSQL_VERSION_ID >= 110400))
+using mrn_handler_referenced_by_foreign_key_bool = bool;
+#  define MRN_HANDLER_REFERENCED_BY_FOREIGN_KEY_CONST_NOEXCEPT const noexcept
+#else
+using mrn_handler_referenced_by_foreign_key_bool = uint;
+#  define MRN_HANDLER_REFERENCED_BY_FOREIGN_KEY_CONST_NOEXCEPT
+#endif
+
 class ha_mroonga;
 
 /* structs */
@@ -592,12 +608,18 @@ public:
 
   THD* current_thread();
 
-  const char* table_type() const; // required
-  const char* index_type(uint inx);
+#ifdef MRN_HANDLER_HAVE_TABLE_TYPE
+  const char* table_type() const override;
+#endif
+#ifdef MRN_HANDLER_HAVE_INDEX_TYPE
+  const char* index_type(uint inx) override;
+#endif
   const char** bas_ext() const; // required
 
-  ulonglong table_flags() const;                                // required
-  ulong index_flags(uint idx, uint part, bool all_parts) const; // required
+  ulonglong table_flags() const override; // required
+  ulong index_flags(uint idx,
+                    uint part,
+                    bool all_parts) const override; // required
 
   // required
   int create(const char* name,
@@ -607,9 +629,9 @@ public:
              ,
              dd::Table* table_def
 #endif
-             ) mrn_override;
+             ) override;
 #ifdef MRN_HANDLER_HAVE_GET_SE_PRIVATE_DATA
-  bool get_se_private_data(dd::Table* dd_table, bool reset) mrn_override;
+  bool get_se_private_data(dd::Table* dd_table, bool reset) override;
 #endif
   // required
   int open(const char* name,
@@ -619,95 +641,94 @@ public:
            ,
            const dd::Table* table_def
 #endif
-           ) mrn_override;
-  int info(uint flag); // required
+           ) override;
+  int info(uint flag) override; // required
 
-  uint lock_count() const;
+  uint lock_count() const override;
   THR_LOCK_DATA** store_lock(THD* thd, // required
                              THR_LOCK_DATA** to,
-                             enum thr_lock_type lock_type);
-  int external_lock(THD* thd, int lock_type);
+                             enum thr_lock_type lock_type) override;
+  int external_lock(THD* thd, int lock_type) override;
 
-  int rnd_init(bool scan); // required
-  int rnd_end();
-  void position(const uchar* record); // required
-  int extra_opt(enum ha_extra_function operation, ulong cache_size);
+  int rnd_init(bool scan) override; // required
+  int rnd_end() override;
+  void position(const uchar* record) override; // required
+  int extra_opt(enum ha_extra_function operation, ulong cache_size) override;
 
   int delete_table(const char* name
 #ifdef MRN_HANDLER_DELETE_TABLE_HAVE_TABLE_DEFINITION
                    ,
                    const dd::Table* table_def
 #endif
-                   ) mrn_override;
-  int write_row(mrn_write_row_buf_t buf) mrn_override;
+                   ) override;
+  int write_row(mrn_write_row_buf_t buf) override;
   int update_row(const uchar* old_data,
-                 mrn_update_row_new_data_t new_data) mrn_override;
-  int delete_row(const uchar* buf);
+                 mrn_update_row_new_data_t new_data) override;
+  int delete_row(const uchar* buf) override;
 
-  uint max_supported_record_length() const;
-  uint max_supported_keys() const;
-  uint max_supported_key_parts() const;
-  uint max_supported_key_length() const;
+  uint max_supported_record_length() const override;
+  uint max_supported_keys() const override;
+  uint max_supported_key_parts() const override;
+  uint max_supported_key_length() const override;
   uint max_supported_key_part_length(
 #ifdef MRN_HANDLER_MAX_SUPPORTED_KEY_PART_LENGTH_HAVE_CREATE_INFO
     HA_CREATE_INFO* create_info
 #endif
-  ) const mrn_override;
+  ) const override;
 
 #ifdef MRN_HANDLER_RECORDS_IN_RANGE_HAVE_PAGE_RANGE
   ha_rows records_in_range(uint inx,
                            const key_range* min_key,
                            const key_range* max_key,
-                           page_range* pages) mrn_override;
+                           page_range* pages) override;
 #else
-  ha_rows records_in_range(uint inx,
-                           key_range* min_key,
-                           key_range* max_key) mrn_override;
+  ha_rows
+  records_in_range(uint inx, key_range* min_key, key_range* max_key) override;
 #endif
-  int index_init(uint idx, bool sorted);
-  int index_end();
+  int index_init(uint idx, bool sorted) override;
+  int index_end() override;
 #ifdef MRN_HANDLER_HAVE_INDEX_READ_LAST_MAP
   int index_read_last_map(uchar* buf,
                           const uchar* key,
-                          key_part_map keypart_map);
+                          key_part_map keypart_map) override;
 #endif
-  int index_next_same(uchar* buf, const uchar* key, uint keylen);
+  int index_next_same(uchar* buf, const uchar* key, uint keylen) override;
 
-  int ft_init();
-  FT_INFO* ft_init_ext(uint flags, uint inx, String* key);
-  int ft_read(uchar* buf);
+  int ft_init() override;
+  FT_INFO* ft_init_ext(uint flags, uint inx, String* key) override;
+  int ft_read(uchar* buf) override;
 
   const Item* cond_push(const Item* cond
 #ifdef MRN_HANDLER_COND_PUSH_HAVE_OTHER_TABLES_OK
                         ,
                         bool other_tables_ok
 #endif
-                        ) mrn_override;
+                        ) override;
 #ifdef MRN_HANDLER_HAVE_COND_POP
-  void cond_pop() mrn_override;
+  void cond_pop() override;
 #endif
 
-  int reset();
+  int reset() override;
 
-  handler* clone(const char* name, MEM_ROOT* mem_root) mrn_override;
-  void print_error(int error, myf flag) mrn_override;
-  bool get_error_message(int error, String* buffer) mrn_override;
+  handler* clone(const char* name, MEM_ROOT* mem_root) override;
+  void print_error(int error, myf flag) override;
+  bool get_error_message(int error, String* buffer) override;
 #ifdef MRN_HANDLER_HAVE_GET_FOREIGN_DUP_KEY
   bool get_foreign_dup_key(char* child_table_name,
                            uint child_table_name_len,
                            char* child_key_name,
-                           uint child_key_name_len) mrn_override;
+                           uint child_key_name_len) override;
 #endif
-  void change_table_ptr(TABLE* table_arg, TABLE_SHARE* share_arg) mrn_override;
-  mrn_io_and_cpu_cost scan_time() mrn_override;
+  void change_table_ptr(TABLE* table_arg, TABLE_SHARE* share_arg) override;
+  mrn_io_and_cpu_cost scan_time() override;
 #if defined(MRN_HANDLER_HAVE_READ_TIME) && defined(MRN_ENABLE_WRAPPER_MODE)
-  double read_time(uint index, uint ranges, ha_rows rows) mrn_override;
+  double read_time(uint index, uint ranges, ha_rows rows) override;
 #endif
 #ifdef MRN_HANDLER_HAVE_GET_MEMORY_BUFFER_SIZE
-  longlong get_memory_buffer_size() const mrn_override;
+  longlong get_memory_buffer_size() const override;
 #endif
 #ifdef MRN_HANDLER_HAVE_TABLE_CACHE_TYPE
-  uint8 table_cache_type();
+  uint8 table_cache_type() override;
 #endif
 #ifdef MRN_ENABLE_WRAPPER_MODE
   ha_rows multi_range_read_info_const(uint keyno,
@@ -719,7 +740,7 @@ public:
 #  ifdef MRN_HANDLER_HAVE_MULTI_RANGE_READ_INFO_CONST_LIMIT
                                       ha_rows limit,
 #  endif
-                                      Cost_estimate* cost) mrn_override;
+                                      Cost_estimate* cost) override;
 #endif
   ha_rows multi_range_read_info(uint keyno,
                                 uint n_ranges,
@@ -729,46 +750,46 @@ public:
 #endif
                                 uint* bufsz,
                                 uint* flags,
-                                Cost_estimate* cost) mrn_override;
+                                Cost_estimate* cost) override;
   int multi_range_read_init(RANGE_SEQ_IF* seq,
                             void* seq_init_param,
                             uint n_ranges,
                             uint mode,
-                            HANDLER_BUFFER* buf) mrn_override;
-  int multi_range_read_next(range_id_t* range_info) mrn_override;
+                            HANDLER_BUFFER* buf) override;
+  int multi_range_read_next(range_id_t* range_info) override;
 #ifdef MRN_HANDLER_START_BULK_INSERT_HAS_FLAGS
-  void start_bulk_insert(ha_rows rows, uint flags);
+  void start_bulk_insert(ha_rows rows, uint flags) override;
 #else
-  void start_bulk_insert(ha_rows rows);
+  void start_bulk_insert(ha_rows rows) override;
 #endif
-  int end_bulk_insert();
+  int end_bulk_insert() override;
 #ifdef MRN_HANDLER_HAVE_GET_SE_PRIVATE_DATA
   bool upgrade_table(THD* thd,
                      const char* db_name,
                      const char* table_name,
-                     dd::Table* dd_table) mrn_override;
+                     dd::Table* dd_table) override;
 #endif
-  int delete_all_rows();
+  int delete_all_rows() override;
   int truncate(
 #ifdef MRN_HANDLER_TRUNCATE_HAVE_TABLE_DEFINITION
     dd::Table* table_def
 #endif
-    ) mrn_override;
+    ) override;
 #ifdef MRN_HANDLER_HAVE_KEYS_TO_USE_FOR_SCANNING
-  const key_map* keys_to_use_for_scanning();
+  const key_map* keys_to_use_for_scanning() override;
 #endif
-  ha_rows estimate_rows_upper_bound() mrn_override;
+  ha_rows estimate_rows_upper_bound() override;
 #ifdef MRN_HANDLER_HAVE_GET_REAL_ROW_TYPE
   enum row_type
-  get_real_row_type(const HA_CREATE_INFO* create_info) const mrn_override;
+  get_real_row_type(const HA_CREATE_INFO* create_info) const override;
 #endif
 #ifdef MRN_HANDLER_HAVE_GET_DEFAULT_INDEX_ALGORITHM
-  enum ha_key_alg get_default_index_algorithm() const mrn_override;
+  enum ha_key_alg get_default_index_algorithm() const override;
 #endif
 #ifdef MRN_HANDLER_HAVE_IS_INDEX_ALGORITHM_SUPPORTED
-  bool is_index_algorithm_supported(enum ha_key_alg key_alg) const mrn_override;
+  bool is_index_algorithm_supported(enum ha_key_alg key_alg) const override;
 #endif
-  void update_create_info(HA_CREATE_INFO* create_info);
+  void update_create_info(HA_CREATE_INFO* create_info) override;
   int rename_table(const char* from,
                    const char* to
 #ifdef MRN_HANDLER_RENAME_TABLE_HAVE_TABLE_DEFINITION
@@ -776,93 +797,94 @@ public:
                    const dd::Table* from_table_def,
                    dd::Table* to_table_def
 #endif
-                   ) mrn_override;
-  bool is_crashed() const;
-  bool auto_repair(int error) const;
-  bool auto_repair() const;
-  int disable_indexes(MRN_HANDLER_ENABLE_INDEXES_PARAMETERS);
-  int enable_indexes(MRN_HANDLER_ENABLE_INDEXES_PARAMETERS);
-  int check(THD* thd, HA_CHECK_OPT* check_opt);
-  int repair(THD* thd, HA_CHECK_OPT* check_opt);
-  bool check_and_repair(THD* thd);
-  int analyze(THD* thd, HA_CHECK_OPT* check_opt);
-  int optimize(THD* thd, HA_CHECK_OPT* check_opt);
+                   ) override;
+  bool is_crashed() const override;
+  bool auto_repair(int error) const MRN_HANDLER_AUTO_REPAIR_ERROR_OVERRIDE;
+  bool auto_repair() const MRN_HANDLER_AUTO_REPAIR_VOID_OVERRIDE;
+  int disable_indexes(MRN_HANDLER_ENABLE_INDEXES_PARAMETERS) override;
+  int enable_indexes(MRN_HANDLER_ENABLE_INDEXES_PARAMETERS) override;
+  int check(THD* thd, HA_CHECK_OPT* check_opt) override;
+  int repair(THD* thd, HA_CHECK_OPT* check_opt) override;
+  bool check_and_repair(THD* thd) override;
+  int analyze(THD* thd, HA_CHECK_OPT* check_opt) override;
+  int optimize(THD* thd, HA_CHECK_OPT* check_opt) override;
   bool is_fatal_error(int error_num
 #ifdef MRN_HANDLER_IS_FATAL_ERROR_HAVE_FLAGS
                       ,
                       uint flags
 #endif
-                      ) mrn_override;
+                      ) override;
   bool check_if_incompatible_data(HA_CREATE_INFO* create_info,
-                                  uint table_changes);
-  enum_alter_inplace_result check_if_supported_inplace_alter(
-    TABLE* altered_table, Alter_inplace_info* ha_alter_info) mrn_override;
+                                  uint table_changes) override;
+  enum_alter_inplace_result
+  check_if_supported_inplace_alter(TABLE* altered_table,
+                                   Alter_inplace_info* ha_alter_info) override;
   int update_auto_increment();
   void set_next_insert_id(ulonglong id);
   void get_auto_increment(ulonglong offset,
                           ulonglong increment,
                           ulonglong nb_desired_values,
                           ulonglong* first_value,
-                          ulonglong* nb_reserved_values);
+                          ulonglong* nb_reserved_values) override;
   void restore_auto_increment(ulonglong prev_insert_id)
     MRN_HANDLER_RESTORE_AUTO_INCREMENT_OVERRIDE;
 #ifdef MRN_HANDLER_HAVE_RESTORE_AUTO_INCREMENT_NO_ARGUMENT
   void restore_auto_increment() MRN_HANDLER_RESTORE_AUTO_INCREMENT_OVERRIDE;
 #endif
   void release_auto_increment() MRN_HANDLER_RELEASE_AUTO_INCREMENT_OVERRIDE;
-  int check_for_upgrade(HA_CHECK_OPT* check_opt);
+  int check_for_upgrade(HA_CHECK_OPT* check_opt) override;
 #ifdef MRN_HANDLER_HAVE_RESET_AUTO_INCREMENT
-  int reset_auto_increment(ulonglong value);
+  int reset_auto_increment(ulonglong value) override;
 #endif
-  bool was_semi_consistent_read();
-  void try_semi_consistent_read(bool yes);
-  void unlock_row();
-  int start_stmt(THD* thd, thr_lock_type lock_type);
+  bool was_semi_consistent_read() override;
+  void try_semi_consistent_read(bool yes) override;
+  void unlock_row() override;
+  int start_stmt(THD* thd, thr_lock_type lock_type) override;
 
 #ifdef MRN_HANDLER_HAVE_HAS_GAP_LOCKS
-  bool has_gap_locks() const MRN_HANDLER_HAS_GAP_LOCKS_NOEXCEPT mrn_override;
+  bool has_gap_locks() const MRN_HANDLER_HAS_GAP_LOCKS_NOEXCEPT override;
 #endif
 
 protected:
 #ifdef MRN_HANDLER_RECORDS_RETURN_ERROR
-  int records(ha_rows* num_rows) mrn_override;
+  int records(ha_rows* num_rows) override;
 #else
-  ha_rows records() mrn_override;
+  ha_rows records() override;
 #endif
-  int rnd_next(uchar* buf) mrn_override;
-  int rnd_pos(uchar* buf, uchar* pos) mrn_override;
+  int rnd_next(uchar* buf) override;
+  int rnd_pos(uchar* buf, uchar* pos) override;
   int index_read_map(uchar* buf,
                      const uchar* key,
                      key_part_map keypart_map,
-                     enum ha_rkey_function find_flag) mrn_override;
-  int index_next(uchar* buf) mrn_override;
-  int index_prev(uchar* buf) mrn_override;
-  int index_first(uchar* buf) mrn_override;
-  int index_last(uchar* buf) mrn_override;
+                     enum ha_rkey_function find_flag) override;
+  int index_next(uchar* buf) override;
+  int index_prev(uchar* buf) override;
+  int index_first(uchar* buf) override;
+  int index_last(uchar* buf) override;
 #ifdef MRN_HANDLER_HAVE_PRIMARY_KEY_IS_CLUSTERED
   bool primary_key_is_clustered()
-    MRN_HANDLER_PRIMARY_KEY_IS_CLUSTERED_CONST mrn_override;
+    MRN_HANDLER_PRIMARY_KEY_IS_CLUSTERED_CONST override;
 #endif
 #ifdef MRN_HANDLER_HAVE_CAN_SWITCH_ENGINES
-  bool can_switch_engines();
+  bool can_switch_engines() override;
 #endif
 #ifdef MRN_HANDLER_HAVE_FOREIGN_KEY_INFO
-  bool is_fk_defined_on_table_or_index(uint index) mrn_override;
-  char* get_foreign_key_create_info() mrn_override;
+  bool is_fk_defined_on_table_or_index(uint index) override;
+  char* get_foreign_key_create_info() override;
   int get_foreign_key_list(THD* thd,
-                           List<FOREIGN_KEY_INFO>* f_key_list) mrn_override;
-  int get_parent_foreign_key_list(THD* thd, List<FOREIGN_KEY_INFO>* f_key_list)
-    mrn_override;
-  mrn_return_type_referenced_by_foreign_key referenced_by_foreign_key()
-    MRN_REFERENCED_BY_FOREIGN_KEY_CONST_NOEXCEPT mrn_override;
-  void free_foreign_key_create_info(char* str) mrn_override;
+                           List<FOREIGN_KEY_INFO>* f_key_list) override;
+  int get_parent_foreign_key_list(THD* thd,
+                                  List<FOREIGN_KEY_INFO>* f_key_list) override;
+  mrn_handler_referenced_by_foreign_key_bool referenced_by_foreign_key()
+    MRN_HANDLER_REFERENCED_BY_FOREIGN_KEY_CONST_NOEXCEPT override;
+  void free_foreign_key_create_info(char* str) override;
 #endif
-  void init_table_handle_for_HANDLER();
+  void init_table_handle_for_HANDLER() override;
 #ifdef MRN_HANDLER_NEED_OVERRIDE_UNBIND_PSI
-  void unbind_psi() mrn_override;
+  void unbind_psi() override;
 #endif
 #ifdef MRN_HANDLER_NEED_OVERRIDE_REBIND_PSI
-  void rebind_psi() mrn_override;
+  void rebind_psi() override;
 #endif
   bool prepare_inplace_alter_table(TABLE* altered_table,
                                    Alter_inplace_info* ha_alter_info
@@ -871,7 +893,7 @@ protected:
                                    const dd::Table* old_table_def,
                                    dd::Table* new_table_def
 #endif
-                                   ) mrn_override;
+                                   ) override;
   bool inplace_alter_table(TABLE* altered_table,
                            Alter_inplace_info* ha_alter_info
 #ifdef MRN_HANDLER_INPLACE_ALTER_TABLE_HAVE_TABLE_DEFINITION
@@ -879,7 +901,7 @@ protected:
                            const dd::Table* old_table_def,
                            dd::Table* new_table_def
 #endif
-                           ) mrn_override;
+                           ) override;
   bool commit_inplace_alter_table(TABLE* altered_table,
                                   Alter_inplace_info* ha_alter_info,
                                   bool commit
@@ -888,21 +910,21 @@ protected:
                                   const dd::Table* old_table_def,
                                   dd::Table* new_table_def
 #endif
-                                  ) mrn_override;
+                                  ) override;
 #ifdef MRN_HANDLER_HAVE_NOTIFY_TABLE_CHANGED
   void notify_table_changed(
 #  ifdef MRN_HANDLER_NOTIFY_TABLE_CHANGED_HAVE_ALTER_INPLACE_INFO
     Alter_inplace_info* ha_alter_info
 #  endif
-    ) mrn_override;
+    ) override;
 #endif
 
 #if defined(MRN_HANDLER_HAVE_KEYREAD_TIME) && defined(MRN_ENABLE_WRAPPER_MODE)
   IO_AND_CPU_COST keyread_time(uint index,
                                ulong ranges,
                                ha_rows rows,
-                               ulonglong blocks) mrn_override;
-  IO_AND_CPU_COST rnd_pos_time(ha_rows rows) mrn_override;
+                               ulonglong blocks) override;
+  IO_AND_CPU_COST rnd_pos_time(ha_rows rows) override;
 #endif
 
 private:
@@ -964,8 +986,8 @@ private:
   int generic_geo_open_cursor(const uchar* key,
                               enum ha_rkey_function find_flag);
 
-  int close() mrn_override;
-  int extra(enum ha_extra_function operation) mrn_override;
+  int close() override;
+  int extra(enum ha_extra_function operation) override;
 
   bool is_dry_write();
   bool is_enable_optimization();
@@ -2012,11 +2034,11 @@ private:
   int storage_get_parent_foreign_key_list(THD* thd,
                                           List<FOREIGN_KEY_INFO>* f_key_list);
 #  ifdef MRN_ENABLE_WRAPPER_MODE
-  mrn_return_type_referenced_by_foreign_key wrapper_referenced_by_foreign_key()
-    MRN_REFERENCED_BY_FOREIGN_KEY_CONST_NOEXCEPT;
+  mrn_handler_referenced_by_foreign_key_bool wrapper_referenced_by_foreign_key()
+    MRN_HANDLER_REFERENCED_BY_FOREIGN_KEY_CONST_NOEXCEPT;
 #  endif
-  mrn_return_type_referenced_by_foreign_key storage_referenced_by_foreign_key()
-    MRN_REFERENCED_BY_FOREIGN_KEY_CONST_NOEXCEPT;
+  mrn_handler_referenced_by_foreign_key_bool storage_referenced_by_foreign_key()
+    MRN_HANDLER_REFERENCED_BY_FOREIGN_KEY_CONST_NOEXCEPT;
 #  ifdef MRN_ENABLE_WRAPPER_MODE
   void wrapper_free_foreign_key_create_info(char* str);
 #  endif
