@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
   Copyright(C) 2011-2013  Kentoku SHIBA
-  Copyright(C) 2011-2022  Sutou Kouhei <kou@clear-code.com>
+  Copyright(C) 2011-2023  Sutou Kouhei <kou@clear-code.com>
   Copyright(C) 2020-2021  Horimoto Yasuhiro <horimoto@clear-code.com>
 
   This library is free software; you can redistribute it and/or
@@ -50,27 +50,6 @@
 
 #include <cstring>
 
-#ifdef MRN_MARIADB_P
-#  if MYSQL_VERSION_ID >= 100300
-     typedef LEX_CSTRING mrn_resolve_name;
-#  else
-     typedef LEX_STRING mrn_resolve_name;
-#  endif
-#  if MYSQL_VERSION_ID >= 100100
-#    define MRN_HA_RESOLVE_BY_NAME(name) ha_resolve_by_name(NULL, (name), true)
-#  else
-#    define MRN_HA_RESOLVE_BY_NAME(name) ha_resolve_by_name(NULL, (name))
-#  endif
-#else
-#  if MYSQL_VERSION_ID >= 80017
-     using mrn_resolve_name = LEX_CSTRING;
-#    define MRN_HA_RESOLVE_BY_NAME(name) ha_resolve_by_name(NULL, (name), true)
-#  else
-     typedef LEX_STRING mrn_resolve_name;
-#    define MRN_HA_RESOLVE_BY_NAME(name) ha_resolve_by_name(NULL, (name), true)
-#  endif
-#endif
-
 #ifdef MRN_MARIADB_P
 #  define MRN_PLUGIN_DATA(plugin, type) plugin_data(plugin, type)
 #else
@@ -89,10 +68,6 @@
 extern mrn_table_def_cache_type *mrn_table_def_cache;
 #endif
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 #ifdef HAVE_PSI_INTERFACE
 #  ifdef WIN32
 #    ifdef MRN_TABLE_SHARE_HAVE_LOCK_SHARE
@@ -517,7 +492,7 @@ int mrn_parse_table_param(MRN_SHARE *share, TABLE *table)
 #ifdef MRN_ENABLE_WRAPPER_MODE
   if (share->engine)
   {
-    mrn_resolve_name engine_name;
+    LEX_CSTRING engine_name;
     if (
       (
         share->engine_length == MRN_DEFAULT_LEN &&
@@ -534,7 +509,7 @@ int mrn_parse_table_param(MRN_SHARE *share, TABLE *table)
     } else {
       engine_name.str = share->engine;
       engine_name.length = share->engine_length;
-      if (!(share->plugin = MRN_HA_RESOLVE_BY_NAME(&engine_name)))
+      if (!(share->plugin = ha_resolve_by_name(nullptr, &engine_name, true)))
       {
         my_error(ER_UNKNOWN_STORAGE_ENGINE, MYF(0), share->engine);
         error = ER_UNKNOWN_STORAGE_ENGINE;
@@ -889,6 +864,9 @@ MRN_SHARE *mrn_get_share(const char *table_name, TABLE *table, int *error)
       wrap_table_share->keys = share->wrap_keys;
       wrap_table_share->key_info = share->wrap_key_info;
       wrap_table_share->primary_key = share->wrap_primary_key;
+#if defined(MRN_MARIADB_P) && (MYSQL_VERSION_ID >= 110800)
+      wrap_table_share->total_keys = share->wrap_keys;
+#endif
       wrap_table_share->keys_in_use.init(share->wrap_keys);
       wrap_table_share->keys_for_keyread.init(share->wrap_keys);
 #  ifdef MRN_TABLE_SHARE_HAVE_LOCK_SHARE
@@ -952,12 +930,15 @@ error_alloc_share:
   DBUG_RETURN(NULL);
 }
 
-int mrn_free_share(MRN_SHARE *share)
+int mrn_free_share(MRN_SHARE *share, bool free_long_term_share)
 {
   MRN_DBUG_ENTER_FUNCTION();
   mrn::Lock lock(&mrn_open_tables_mutex);
   if (!--share->use_count)
   {
+    if (free_long_term_share) {
+      mrn_free_long_term_share(share->long_term_share);
+    }
     grn_hash_delete(&mrn_ctx,
                     mrn_open_tables,
                     share->table_name,
@@ -989,18 +970,12 @@ TABLE_SHARE *mrn_get_table_share(TABLE_LIST *table_list, int *error)
   TABLE_SHARE *share;
   THD *thd = current_thd;
   MRN_DBUG_ENTER_FUNCTION();
-#if defined(MRN_HAVE_TDC_ACQUIRE_SHARE) &&      \
-  !defined(MRN_TDC_ACQUIRE_SHARE_REQUIRE_KEY)
+#if defined(MRN_HAVE_TDC_ACQUIRE_SHARE)
   share = tdc_acquire_share(thd, table_list, GTS_TABLE);
 #else
   uint key_length;
-#  ifdef MRN_HAVE_GET_TABLE_DEF_KEY
   const char *key;
   key_length = get_table_def_key(table_list, &key);
-#  else
-  char key[MAX_DBKEY_LENGTH];
-  key_length = create_table_def_key(thd, key, table_list, false);
-#  endif
 #  ifdef MRN_HAVE_TABLE_DEF_CACHE
 #    ifdef MRN_TABLE_DEF_CACHE_TYPE_IS_MAP
   share = get_table_share(thd,
@@ -1039,13 +1014,8 @@ TABLE_SHARE *mrn_create_tmp_table_share(TABLE_LIST *table_list,
   THD *thd = current_thd;
 
   MRN_DBUG_ENTER_FUNCTION();
-#ifdef MRN_HAVE_GET_TABLE_DEF_KEY
   const char *key;
   key_length = get_table_def_key(table_list, &key);
-#else
-  char key[MAX_DBKEY_LENGTH];
-  key_length = create_table_def_key(thd, key, table_list, false);
-#endif
   share = mrn_alloc_table_share(table_list, key, key_length);
   if (!share)
   {
@@ -1129,14 +1099,16 @@ void mrn_set_bitmap_by_key(MY_BITMAP *map, KEY *key_info)
   DBUG_VOID_RETURN;
 }
 
-st_mrn_slot_data *mrn_get_slot_data(THD *thd, bool can_create)
+mrn::SlotData *mrn_get_slot_data(THD *thd, bool can_create)
 {
   MRN_DBUG_ENTER_FUNCTION();
-  st_mrn_slot_data *slot_data =
-    static_cast<st_mrn_slot_data *>(thd_get_ha_data(thd, mrn_hton_ptr));
-  if (slot_data == NULL) {
-    slot_data =
-      static_cast<st_mrn_slot_data *>(malloc(sizeof(st_mrn_slot_data)));
+  if (!thd) {
+    DBUG_RETURN(nullptr);
+  }
+  mrn::SlotData *slot_data =
+    static_cast<mrn::SlotData *>(thd_get_ha_data(thd, mrn_hton_ptr));
+  if (!slot_data && can_create) {
+    slot_data = new mrn::SlotData();
     slot_data->last_insert_record_id = GRN_ID_NIL;
 #ifdef MRN_ENABLE_WRAPPER_MODE
     slot_data->first_wrap_hton = NULL;
@@ -1145,7 +1117,6 @@ st_mrn_slot_data *mrn_get_slot_data(THD *thd, bool can_create)
     slot_data->disable_keys_create_info = NULL;
     slot_data->alter_connect_string = NULL;
     slot_data->alter_comment = NULL;
-    mrn_thd_set_ha_data(thd, mrn_hton_ptr, slot_data);
     {
       mrn::Lock lock(&mrn_allocated_thds_mutex);
       if (grn_hash_add(&mrn_ctx,
@@ -1154,46 +1125,11 @@ st_mrn_slot_data *mrn_get_slot_data(THD *thd, bool can_create)
                        sizeof(thd),
                        NULL,
                        NULL) == GRN_ID_NIL) {
-        free(slot_data);
+        delete slot_data;
         DBUG_RETURN(NULL);
       }
     }
+    mrn_thd_set_ha_data(thd, mrn_hton_ptr, slot_data);
   }
   DBUG_RETURN(slot_data);
 }
-
-void mrn_clear_slot_data(THD *thd)
-{
-  MRN_DBUG_ENTER_FUNCTION();
-  st_mrn_slot_data *slot_data = mrn_get_slot_data(thd, false);
-  if (slot_data) {
-#ifdef MRN_ENABLE_WRAPPER_MODE
-    if (slot_data->first_wrap_hton) {
-      st_mrn_wrap_hton *tmp_wrap_hton;
-      st_mrn_wrap_hton *wrap_hton = slot_data->first_wrap_hton;
-      while (wrap_hton)
-      {
-        tmp_wrap_hton = wrap_hton->next;
-        free(wrap_hton);
-        wrap_hton = tmp_wrap_hton;
-      }
-      slot_data->first_wrap_hton = NULL;
-    }
-#endif
-    slot_data->alter_create_info = NULL;
-    slot_data->disable_keys_create_info = NULL;
-    if (slot_data->alter_connect_string) {
-      my_free(slot_data->alter_connect_string);
-      slot_data->alter_connect_string = NULL;
-    }
-    if (slot_data->alter_comment) {
-      my_free(slot_data->alter_comment);
-      slot_data->alter_comment = NULL;
-    }
-  }
-  DBUG_VOID_RETURN;
-}
-
-#ifdef __cplusplus
-}
-#endif
